// Device.cpp: implementation of the CInst class.
//
//////////////////////////////////////////////////////////////////////

#include "stdafx.h"
#include <sys/types.h>
#include <sys/timeb.h>
#include <time.h>
#include <afxtempl.h>
#include "crobot.h"
#include "crobotinternet.h"
#include "HotOffThePress.h"
#include "DeviceIDDlg.h"
#include "SouceDlg.h"
#include "TimeTickerDlg.h"
#include "NewzDlg.h"
#include "FinancialDlg.h"
#include "AlertsDlg.h"
#include "UpdatesDlg.h"
#include "NoDeviceDlg.h"
#include "DownloadDlg.h"
#include "FindNumDlg.h"
#include "Keywords.h"
#include "CalibNumDlg.h"
#include "AlignDlg.h"
#include "Ports.h"
#include "Device.h"
#include "DevicesDlg.h"
#include "YahooTicker.h"
#include "CalibNumDlg.h"
#include "FindText.h"
#include "WebNumDial.h"
#include "WebTextDial.h"
#include "Xmitter.h"

// Serialization number version number
#define VERSIONNO 1

#ifdef _DEBUG
#undef THIS_FILE
static char THIS_FILE[]=__FILE__;
#define new DEBUG_NEW
#endif

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

IMPLEMENT_SERIAL( CDevice, CObject, VERSIONNO );

// CDevice is a generic instrument

CDevice::CDevice()
{
	m_DeviceName.LoadString( IDS_NODEVICES );
	m_DeviceType = DT_NULLDEVICE;
	m_ResultText.LoadString( IDS_NORESULTTEXT );
	m_strDeviceID = "";
	m_strURL = "";
	m_iUpdate = 300;
	strcpy( m_chid , "9876");
	m_bSleep = false;
	m_Wake = CTime::CTime( 1990, 1, 1, 8, 0, 0 );
	m_Sleep = CTime::CTime( 1990, 1, 1, 22, 0, 0 );
}

CDevice::CDevice( CDevice& dv)
{
	*this = dv;
}

void CDevice::operator=(CDevice& yt)
{
	m_DeviceName = yt.m_DeviceName;
	m_DeviceType = yt.m_DeviceType;
	m_ResultText = yt.m_ResultText;
	m_strDeviceID = yt.m_strDeviceID;
	m_strURL = yt.m_strURL;
	m_iUpdate = yt.m_iUpdate;
	m_bSleep = yt.m_bSleep;
	m_Wake = yt.m_Wake;
	m_Sleep = yt.m_Sleep;
	strcpy( m_chid , yt.m_chid);

//	return yt;
}

CDevice::~CDevice()
{

}

CString& CDevice::getDeviceName()
{
	return m_DeviceName;
}

void CDevice::setDeviceName(LPCSTR str)
{
	m_DeviceName = str;
}

DeviceType CDevice::getDeviceType()
{
	return m_DeviceType;
}

void CDevice::setDeviceType(DeviceType Type)
{
	m_DeviceType = Type;
}

void CDevice::AddPages()
{
	switch (m_DeviceType)
	{
	case DT_NULLDEVICE:
		// nothing to add
		break;

	case DT_XMITTER:
		((CXmitter*)this)->AddPages();
		break;

	case DT_YAHOOTICKER:
		((CYahooTicker*)this)->AddPages();
		break;

	case DT_WEBNUMDIAL:
		((CWebNumDial*)this)->AddPages();
		break;

	case DT_WEBTEXTDIAL:
		((CWebTextDial*)this)->AddPages();
		break;

	default:
		AfxMessageBox("Error In Device.cpp addpages switch");
	}
}

void CDevice::RemovePages()
{
	switch (m_DeviceType)
	{
	case DT_NULLDEVICE:
		// nothing to remove
		break;

	case DT_XMITTER:
		((CXmitter*)this)->RemovePages();
		break;

	case DT_YAHOOTICKER:
		((CYahooTicker*)this)->RemovePages();
		break;

	case DT_WEBNUMDIAL:
		((CWebNumDial*)this)->RemovePages();
		break;

	case DT_WEBTEXTDIAL:
		((CWebTextDial*)this)->RemovePages();
		break;

	default:
		AfxMessageBox("Error In Device.cpp RemovePages switch");
	}
}

// Serialization
void CDevice::Serialize(CArchive &ar)
{
	CObject::Serialize( ar );

	if (ar.IsStoring())
	{
		ar << m_DeviceName;
		ar << (long)m_DeviceType;
		ar << m_chid[0];
		ar << m_chid[1];
		ar << m_chid[2];
		ar << m_chid[3];

		ar << m_strDeviceID;
		ar << m_strURL;
		ar << m_iUpdate;

		ar << m_bSleep;
		ar << m_Wake;
		ar << m_Sleep;
	}
	else
	{
		ar >> m_DeviceName;
		long iDev;
		ar >> iDev;
		m_DeviceType = (DeviceType) iDev;
		ar >> m_chid[0];
		ar >> m_chid[1];
		ar >> m_chid[2];
		ar >> m_chid[3];
		m_chid[4] = '\0';

		ar >> m_strDeviceID;
		ar >> m_strURL;
		ar >> m_iUpdate;

		ar >> m_bSleep;
		ar >> m_Wake;
		ar >> m_Sleep;
	}

}


void CDevice::WebUpdate()
{
	switch (m_DeviceType)
	{
	case DT_NULLDEVICE:
	case DT_XMITTER:
		// nothing to do
		return;

	case DT_YAHOOTICKER:
		((CYahooTicker*)this)->WebUpdate();
		break;

	case DT_WEBNUMDIAL:
		((CWebNumDial*)this)->WebUpdate();
		break;

	case DT_WEBTEXTDIAL:
		((CWebTextDial*)this)->WebUpdate();
		break;

	default:
		AfxMessageBox("Error In Device.cpp WebUpdate switch");
	}
}

void CDevice::UpdateNow()
{
	switch (m_DeviceType)
	{
	case DT_NULLDEVICE:
	case DT_XMITTER:
		// nothing to do
		return;

	case DT_YAHOOTICKER:
		((CYahooTicker*)this)->UpdateNow();
		break;

	case DT_WEBNUMDIAL:
		((CWebNumDial*)this)->UpdateNow();
		break;

	case DT_WEBTEXTDIAL:
		((CWebTextDial*)this)->UpdateNow();
		break;

	default:
		AfxMessageBox("Error In Device.cpp UpdateNow switch");
	}
}


void CDevice::AlignMode( bool bMode )
{
}

void CDevice::AlignLeft( )
{
}

void CDevice::AlignRight( )
{
}

void CDevice::AlignFLeft( )
{
}

void CDevice::AlignFRight( )
{
}

void CDevice::TransmitMessage( )
{
	int Com = ((CHotOffThePressApp*)AfxGetApp())->m_iComPort;
	if (Com==0) return;

	switch (m_DeviceType)
	{
	case DT_NULLDEVICE:
	case DT_XMITTER:
		// nothing to do
		return;

	case DT_YAHOOTICKER:
		((CYahooTicker*)this)->TransmitMessage();
		break;

	case DT_WEBNUMDIAL:
		((CWebNumDial*)this)->TransmitMessage();
		break;

	case DT_WEBTEXTDIAL:
		((CWebTextDial*)this)->TransmitMessage();
		break;

	default:
		AfxMessageBox("Error In Device.cpp TransmitMessage switch");
	}
}

