// Threads.cpp: implementation of functions requiring acces to global data for thread management
//
// No associated header file
//
//////////////////////////////////////////////////////////////////////

#include "stdafx.h"
#include <sys/types.h>
#include <sys/timeb.h>
#include <time.h>
#include <afxtempl.h>
#include <afxmt.h>
#include <afxwin.h>
#include "crobot.h"
#include "crobotinternet.h"
#include "HotOffThePress.h"
#include "DeviceIDDlg.h"
#include "SouceDlg.h"
#include "TimeTickerDlg.h"
#include "NewzDlg.h"
#include "FinancialDlg.h"
#include "AlertsDlg.h"
#include "UpdatesDlg.h"
#include "NoDeviceDlg.h"
#include "Ports.h"
#include "Device.h"
#include "DevicesDlg.h"
#include "DownloadDlg.h"
#include "FindNumDlg.h"
#include "CalibNumDlg.h"
#include "AlignDlg.h"
#include "Keywords.h"
#include "YahooTicker.h"
#include "FindText.h"
#include "WebTextDial.h"
#include "WebNumDial.h"

// Global thread data
#define NTHREAD 100
struct ThreadData {
	bool bInUse;
	bool bFinished;
	CString sURL;
	CString sResult;
	long lResult;
	CDevice* pDev;			// This is pointer to original device, which might be deleted
	CDevice* pLocalDev;		// This is copy of device which will not be deleted until bFinished
};

ThreadData Thread[NTHREAD];


//////////////////////////////////////////////////////////////////////

// Initialize global data

void CHotOffThePressApp::InitializeThreads()
{
	for (int i=0; i<NTHREAD; i++)
	{
		Thread[i].bInUse = false;
		Thread[i].bFinished = false;
		Thread[i].sURL = "";
		Thread[i].sResult = "";
		Thread[i].lResult = 0;
		Thread[i].pDev = NULL;
		Thread[i].pLocalDev = NULL;
	}
}


UINT WebThread( LPVOID pParam )
{
	// Identify global data
	ThreadData* pThread = (ThreadData*) pParam;

	// Mark as in use (should be already, but...)
	pThread->bInUse = true;

	// Identify internet handle
	CRobotInternet* pNet = &((CHotOffThePressApp*)AfxGetApp())->m_Internet;

	// Diffreent processing for each device type
	switch (pThread->pLocalDev->m_DeviceType)
	{
	case DT_YAHOOTICKER:
		{
		// Get HTML document
		CString sHTML, sErr;
		int iResult;
		pNet->httpGet( pThread->sURL, sHTML, iResult, sErr ); 

		// Parse data
		((CYahooTicker*)pThread->pLocalDev)->ParseWebPage(	pThread->sResult, 
													sHTML, iResult, sErr );
		}
		break;

	case DT_WEBNUMDIAL:
		{
		// Get HTML document
		CString sHTML, sErr;
		int iResult;
		pNet->httpGet( pThread->sURL, sHTML, iResult, sErr ); 

		// Parse data
		((CWebNumDial*)pThread->pLocalDev)->ParseWebPage(	pThread->sResult, 
										pThread->lResult, sHTML, iResult, sErr );
		}
		break;

	case DT_WEBTEXTDIAL:
		{
		// Get HTML document
		CString sHTML, sErr;
		int iResult;
		pNet->httpGet( pThread->sURL, sHTML, iResult, sErr ); 

		// Parse data
		((CWebTextDial*)pThread->pLocalDev)->ParseWebPage(	pThread->sResult, 
										pThread->lResult, sHTML, iResult, sErr );
		}
		break;

	default:
		AfxMessageBox( "Switch error in webthread(), threads.cpp" );
		break;
	}

	// Flag finished
	pThread->bFinished = true;

	return 0;
}


void CYahooTicker::WebUpdate()
{
	int iThread;

	// Check for active for finished thread for this object
	
	// loop through threads looking for one associated with this device
	for ( iThread=0; iThread<NTHREAD; iThread++)
	{
		if (Thread[iThread].pDev == ((CDevice*)this) && Thread[iThread].bInUse )
		{

			// decide wether to save and clear, or abort
			if (Thread[iThread].bFinished)
			{
				// Clear up thread

				// Copy result
				m_TickerText = Thread[iThread].sResult;
				m_ResultText = m_TickerText;

				// Delete local device
				if (Thread[iThread].pLocalDev) delete Thread[iThread].pLocalDev;

				// Nullify Thread
				Thread[iThread].sURL = "";
				Thread[iThread].sResult = "";
				Thread[iThread].lResult = 0;
				Thread[iThread].pDev = NULL;
				Thread[iThread].pLocalDev = NULL;
				Thread[iThread].bFinished = false;
				Thread[iThread].bInUse = false;
			}
			else
			{
				// Already have thread active - go home now
				return;
			}
		}
	}

	// No threads are currently active for this device if reach hree

	// if due for start, start off new thread
	CTime tNow = CTime::GetCurrentTime();
	CTimeSpan tDiff = tNow - m_LastWebUpdate;
	if ( tDiff.GetTotalMinutes() >= m_iUpdate )
	{
		m_LastWebUpdate = tNow;
 
		// loop through threads looking for finished one
		for ( iThread=0; iThread<NTHREAD; iThread++)
		{
			if (!Thread[iThread].bInUse ) break;
		}

		if (iThread<NTHREAD)
		{
			// found thread - use it
			Thread[iThread].sURL = m_strURL;
			Thread[iThread].sResult = "";
			Thread[iThread].lResult = 0;
			Thread[iThread].pDev = (CDevice*) this;
			Thread[iThread].pLocalDev = (CDevice*) new CYahooTicker(*this);
			Thread[iThread].bFinished = false;
			Thread[iThread].bInUse = true;

			AfxBeginThread( WebThread, (LPVOID) &Thread[iThread] );
		}
	}

}

void CWebNumDial::WebUpdate()
{
	int iThread;

	// Check for active for finished thread for this object
	
	// loop through threads looking for one associated with this device
	for ( iThread=0; iThread<NTHREAD; iThread++)
	{
		if (Thread[iThread].pDev == ((CDevice*)this) && Thread[iThread].bInUse )
		{

			// decide wether to save and clear, or abort
			if (Thread[iThread].bFinished)
			{
				// Clear up thread

				// Copy result
				m_ResultText = Thread[iThread].sResult;
				m_iResult = Thread[iThread].lResult;

				// Delete local device
				if (Thread[iThread].pLocalDev) delete Thread[iThread].pLocalDev;

				// Nullify Thread
				Thread[iThread].sURL = "";
				Thread[iThread].sResult = "";
				Thread[iThread].lResult = 0;
				Thread[iThread].pDev = NULL;
				Thread[iThread].pLocalDev = NULL;
				Thread[iThread].bFinished = false;
				Thread[iThread].bInUse = false;
			}
			else
			{
				// Already have thread active - go home now
				return;
			}
		}
	}

	// No threads are currently active for this device if reach hree

	// if due for start, start off new thread
	CTime tNow = CTime::GetCurrentTime();
	CTimeSpan tDiff = tNow - m_LastWebUpdate;
	if ( tDiff.GetTotalMinutes() >= m_iUpdate )
	{
		m_LastWebUpdate = tNow;
 
		// loop through threads looking for finished one
		for ( iThread=0; iThread<NTHREAD; iThread++)
		{
			if (!Thread[iThread].bInUse ) break;
		}

		if (iThread<NTHREAD)
		{
			// found thread - use it
			Thread[iThread].sURL = m_strURL;
			Thread[iThread].sResult = "";
			Thread[iThread].lResult = 0;
			Thread[iThread].pDev = (CDevice*) this;
			Thread[iThread].pLocalDev = (CDevice*) new CWebNumDial(*this);
			Thread[iThread].bFinished = false;
			Thread[iThread].bInUse = true;

			AfxBeginThread( WebThread, (LPVOID) &Thread[iThread] );
		}
	}

}

void CWebTextDial::WebUpdate()
{
	int iThread;

	// Check for active for finished thread for this object
	
	// loop through threads looking for one associated with this device
	for ( iThread=0; iThread<NTHREAD; iThread++)
	{
		if (Thread[iThread].pDev == ((CDevice*)this) && Thread[iThread].bInUse )
		{

			// decide wether to save and clear, or abort
			if (Thread[iThread].bFinished)
			{
				// Clear up thread

				// Copy result
				m_ResultText = Thread[iThread].sResult;
				m_iResult = Thread[iThread].lResult;

				// Delete local device
				if (Thread[iThread].pLocalDev) delete Thread[iThread].pLocalDev;

				// Nullify Thread
				Thread[iThread].sURL = "";
				Thread[iThread].sResult = "";
				Thread[iThread].lResult = 0;
				Thread[iThread].pDev = NULL;
				Thread[iThread].pLocalDev = NULL;
				Thread[iThread].bFinished = false;
				Thread[iThread].bInUse = false;
			}
			else
			{
				// Already have thread active - go home now
				return;
			}
		}
	}

	// No threads are currently active for this device if reach hree

	// if due for start, start off new thread
	CTime tNow = CTime::GetCurrentTime();
	CTimeSpan tDiff = tNow - m_LastWebUpdate;
	if ( tDiff.GetTotalMinutes() >= m_iUpdate )
	{
		m_LastWebUpdate = tNow;
 
		// loop through threads looking for finished one
		for ( iThread=0; iThread<NTHREAD; iThread++)
		{
			if (!Thread[iThread].bInUse ) break;
		}

		if (iThread<NTHREAD)
		{
			// found thread - use it
			Thread[iThread].sURL = m_strURL;
			Thread[iThread].sResult = "";
			Thread[iThread].lResult = 0;
			Thread[iThread].pDev = (CDevice*) this;
			Thread[iThread].pLocalDev = (CDevice*) new CWebTextDial(*this);
			Thread[iThread].bFinished = false;
			Thread[iThread].bInUse = true;

			AfxBeginThread( WebThread, (LPVOID) &Thread[iThread] );
		}
	}

}

