// YahooTicker.cpp: implementation of the CYahooTicker class.
//
//////////////////////////////////////////////////////////////////////

#include "stdafx.h"
#include <sys/types.h>
#include <sys/timeb.h>
#include <time.h>
#include <afxtempl.h>
#include <afxmt.h>
#include <afxwin.h>
#include "crobot.h"
#include "crobotinternet.h"
#include "HotOffThePress.h"
#include "DeviceIDDlg.h"
#include "SouceDlg.h"
#include "TimeTickerDlg.h"
#include "NewzDlg.h"
#include "FinancialDlg.h"
#include "AlertsDlg.h"
#include "UpdatesDlg.h"
#include "NoDeviceDlg.h"
#include "Ports.h"
#include "Device.h"
#include "DevicesDlg.h"
#include "FindNumDlg.h"
#include "CalibNumDlg.h"
#include "YahooTicker.h"

#define VERSIONNO 1

#ifdef _DEBUG
#undef THIS_FILE
static char THIS_FILE[]=__FILE__;
#define new DEBUG_NEW
#endif

// Global HTML code data
#define NUMHTMLCODE 98
#define MAXCODELEN 10

char HTMLCode[NUMHTMLCODE][MAXCODELEN] = {
							"&nbsp;", "&quot;", "&amp;", "&lt;", "&gt;", 
							"&iexcl;", "&cent;", "&pound;", "&yen;", "&brvbar;", 
							"&sect;", "&uml;", "&laquo;", "&ordf;", "&copy;", 
							"&not;", "&shy;", "&reg;", "&hibar;", "&deg;", 
							"&plusmn;", "&sup2;", "&sup3;", "&acute;", "&micro;", 
							"&para;", "&middot;", "&cedil;", "&sup1;", "&ordm;", 
							"&raquo;", "&frac14;", "&frac12;", "&frac34;", "&iquest;", 
							"&Agrave;", "&Aacute;", "&Acirc;", "&Atilde;", "&Aumi;", 
							"&Aring;", "&AElig;", "&Ccedil;", "&Egrave;", "&Eacute;", 
							"&Ecirc;", "&Euml;", "&Igrave;", "&Iacute;", "&Icirc;", 
							"&Iumi;", "&ETH;", "&Dstrok;", "&Ntilde;", "&Ograve;", 
							"&Oacute;", "&Ocirc;", "&Otilde;", "&Oumi;", "&Oslash;", 
							"&Ugrave;", "&Uacute;", "&Ucirc;", "&Uuml;", "&Yacute;", 
							"&THORN;", "&szlig;", "&agrave;", "&aacute;", "&acirc;", 
							"&atilde;", "&aumi;", "&aring;", "&aelig;", "&ccedil;", 
							"&egrave;", "&eacute;", "&ecirc;", "&euml;", "&igrave;", 
							"&iacute;", "&icirc;", "&iuml;", "&eth;", "&ntilde;", 
							"&ograve;", "&oacute;", "&ocirc;", "&otilde;", "&ouml;", 
							"&oslash;", "&ugrave;", "&uacute;", "&ucirc;", "&uuml;", 
							"&yacute;", "&thorn;", "&yuml;" };


char ReplaceChar[NUMHTMLCODE][MAXCODELEN] = {	
						" ", "\"", "&", "<", ">", 
						"!", "c", "(Pounds) ", "Y", "|",  
						"S", "\"", "\"", "(a)", "(c)",  
						"_", "-", "(r)", "_", "deg",  
						"+/-", "(2)", "(3)", "`", "u",  
						"(P)", ".", ",", "(1)", "(0)",  
						"\"", " 1/4", " 1/2", " 3/4", "?",  
						"A", "A", "A", "A", "A",  
						"A", "AE", "C", "E", "E",  
						"E", "E", "I", "I", "I",  
						"I", "D", "D", "N", "O",  
						"O", "O", "O", "O", "O",  
						"U", "U", "U", "U", "Y",  
						"p", "ss", "a", "a", "a",  
						"a", "a", "a", "ae", "c",  
						"e", "e", "e", "e", "i",  
						"i", "i", "i", "d", "n",  
						"o", "o", "o", "o", "o",  
						"o", "u", "u", "u", "u",  
						"y", "p", "y" };





//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

IMPLEMENT_SERIAL( CYahooTicker, CDevice, VERSIONNO );

// CYahooTicker yahoo ticker display

CYahooTicker::CYahooTicker()
{
	// Propeties
	m_DeviceName.LoadString( IDS_MYYAHOOTICKER );
	m_DeviceType = DT_YAHOOTICKER;

	m_strDeviceID.LoadString( IDS_DEFID );
	m_strURL.LoadString( IDS_DEFURL );
	m_bTime = true;
	m_TimeFormatString.LoadString( IDS_FACTORYTIME );
	m_bLostSignal = true;
	m_bHeadlines = true;
	m_bMaxHead = false;
	m_iMaxHead = 5;
	m_bWeather = true;
	m_bFinancial = true;
	m_bPrice = true;
	m_bChange = false;
	m_bPercent = true;
	m_bNoZeroChange = true;
	m_bOmitDigits = true;
	m_strFirstLine.LoadString( IDS_DEFFIRSTFIN ); 
	m_iNumLine = 8;

	m_bCalcPercent = true;
	m_sTriggerText = "NASDAQ";
	m_bBefore = false;
	m_iNumCount = 2;

	m_HiDeg = 300;
	m_HiVal = 5;
	m_LoDeg = 0;
	m_LoVal = -5;

	CString str;
	str.LoadString(IDS_DEFTRIG);
	m_slAlTrigger.AddHead(str);
	str.LoadString(IDS_DEFMESS);
	m_slAlMessage.AddHead(str);
	m_lbAlBeep.AddHead( BeepType_HIGH );
	m_lbAlFlash.AddHead( FT_GREEN );
	m_lbAlFlag.AddHead( 1 );
	m_TickerText.LoadString( IDS_NOTICKERTEXT );
	m_ResultText.LoadString( IDS_NORESULTTEXT );
	m_bTickerTextDataGood = true;
	m_bLeft = false;
	m_ScrollRate = 20;

	m_cflags = 0;

	m_OldBeep = BeepType_NONE;
	m_BeepMin = -1;

	m_Flash = FT_NONE;
	m_Beep = BeepType_NONE;

	// Updates
	CTime t2( 1999, 1, 1, 1, 1, 1 );
	m_LastWebUpdate = t2;
	_ftime( &m_LastTransmission ); 
	m_NextTextPos = 0;
	m_Line[0] = '\0';

}

CYahooTicker::CYahooTicker( CYahooTicker& yt)
{
	// CDevice stuff
	m_DeviceName = yt.m_DeviceName;
	m_DeviceType = yt.m_DeviceType;
	m_ResultText = yt.m_ResultText;
	m_strDeviceID = yt.m_strDeviceID;
	m_strURL = yt.m_strURL;
	m_iUpdate = yt.m_iUpdate;
	m_bSleep = yt.m_bSleep;
	m_Wake = yt.m_Wake;
	m_Sleep = yt.m_Sleep;
	strcpy( m_chid , yt.m_chid);

	m_bCalcPercent = yt.m_bCalcPercent;
	m_sTriggerText = yt.m_sTriggerText;
	m_bBefore = yt.m_bBefore;
	m_iNumCount = yt.m_iNumCount;

	m_HiDeg = yt.m_HiDeg;
	m_HiVal = yt.m_HiVal;
	m_LoDeg = yt.m_LoDeg;
	m_LoVal = yt.m_LoVal;


	m_slAlTrigger.RemoveAll();
	m_slAlMessage.RemoveAll();
	m_lbAlFlash.RemoveAll();
	m_lbAlFlag.RemoveAll();
	m_lbAlBeep.RemoveAll();
	m_slAlTrigger.AddHead( &yt.m_slAlMessage );
	m_slAlMessage.AddHead( &yt.m_slAlMessage);
	m_lbAlBeep.AddHead( &yt.m_lbAlBeep );
	m_lbAlFlash.AddHead( &yt.m_lbAlFlash );
	m_lbAlFlag.AddHead( &yt.m_lbAlFlag );

	// Propeties
	m_bTime = yt.m_bTime;
	m_TimeFormatString = yt.m_TimeFormatString;
	m_bLostSignal = yt.m_bLostSignal;
	m_bHeadlines = yt.m_bHeadlines;
	m_bMaxHead = yt.m_bMaxHead;
	m_iMaxHead = yt.m_iMaxHead;
	m_bWeather = yt.m_bWeather;
	m_bFinancial = yt.m_bFinancial;
	m_bPrice = yt.m_bPrice;
	m_bChange = yt.m_bChange;
	m_bPercent = yt.m_bPercent;
	m_bNoZeroChange = yt.m_bNoZeroChange;
	m_bOmitDigits = yt.m_bOmitDigits;
	m_strFirstLine = yt.m_strFirstLine;
	m_iNumLine = yt.m_iNumLine;
	m_bLeft = yt.m_bLeft;
	m_ScrollRate = yt.m_ScrollRate;

	m_Flash = yt.m_Flash;
	m_Beep = yt.m_Beep;

	m_LastWebUpdate = yt.m_LastWebUpdate;

	m_cflags = yt.m_cflags;
	m_OldBeep = yt.m_OldBeep;
	m_BeepMin = yt.m_BeepMin;


}

CYahooTicker::~CYahooTicker()
{

}


void CYahooTicker::AddPages()
{
	// Add property pages to PropSheet
	CDevicesDlg* pDlg = ((CHotOffThePressApp*)AfxGetApp())->m_pDevicesDlg;
//	pDlg->AddPage( &m_DeviceIDDlg );
	pDlg->AddPage( &m_SourceDlg );
//	pDlg->AddPage( &m_TimeTickerDlg );
//	pDlg->AddPage( &m_NewsDlg );
//	pDlg->AddPage( &m_FinancialDlg );
//	pDlg->AddPage( &m_AlertsDlg );
	pDlg->AddPage( &m_FindNum );
	pDlg->AddPage( &m_CalibNum );
	pDlg->AddPage( &m_UpdatesDlg );

	if (TICKERONLY)
	{
		pDlg->SetActivePage( &m_DeviceIDDlg );
	}

}

void CYahooTicker::RemovePages()
{
	// Add property pages to PropSheet
	CDevicesDlg* pDlg = ((CHotOffThePressApp*)AfxGetApp())->m_pDevicesDlg;
//	pDlg->RemovePage( &m_DeviceIDDlg );
	pDlg->RemovePage( &m_SourceDlg );
//	pDlg->RemovePage( &m_TimeTickerDlg );
//	pDlg->RemovePage( &m_NewsDlg );
//	pDlg->RemovePage( &m_FinancialDlg );
//	pDlg->RemovePage( &m_AlertsDlg );
	pDlg->RemovePage( &m_FindNum );
	pDlg->RemovePage( &m_CalibNum );
	pDlg->RemovePage( &m_UpdatesDlg );
}

void CYahooTicker::UpdateNow()
{
	// Polls web page IN THIS THREAD
	int RetVal;
	CString sDoc, sErr;

	// Poll web page
	((CHotOffThePressApp*)AfxGetApp())->m_Internet.httpGet(m_strURL, sDoc, RetVal, sErr);

	// Translate web page
	ParseWebPage( m_TickerText, sDoc, RetVal, sErr ); 

	m_ResultText = m_TickerText;
	((CHotOffThePressApp*)AfxGetApp())->m_bDataUpdate = true;
}


double FindNumberBackwardsFrom( int * Pos, CString str )
{
	bool bInBrace = false;

	// go back to find a digit 0123456789
	while (*Pos >= 0)
	{
		if (str.GetAt(*Pos) == '>') bInBrace = true;
		if (!bInBrace && strrchr( (const char *) "01234567879", str.GetAt(*Pos) )) break;
		if (str.GetAt(*Pos) == '<') bInBrace = false;
		(*Pos)--;
	}
	if (*Pos < 0) return 0;

	// go back while -.0123456789
	while (*Pos >= 0 && strrchr( (const char *) "-.0123456789", str.GetAt(*Pos) ))
	{
		(*Pos)--;
	}

	// there's your number
	double num = 0;
	sscanf( (LPCTSTR) str.Mid( *Pos+1 ), "%lf", &num );
	return num;
}


double FindNumberForwardsFrom( int * Pos, CString str )
{
	bool bInBrace = false;

	// go forward to find a digit 0123456789
	while (*Pos < str.GetLength() )
	{
		if (str.GetAt(*Pos) == '<') bInBrace = true;
		if (!bInBrace && strrchr( "01234567879", str.GetAt(*Pos) ) ) break;
		if (str.GetAt(*Pos) == '>') bInBrace = false;
		(*Pos)++;
	}
	if (*Pos == str.GetLength()) return 0;

	// check if minus sign immediately before
	if (*Pos > 0 && str.GetAt(*Pos-1)=='-') (*Pos)--; 

	// there's your number
	double num;
	sscanf( (LPCTSTR) str.Mid( *Pos ), "%lf", &num );

	// zoom to end of number
	
	// go back while -.0123456789
	while (*Pos < str.GetLength() && strrchr( (const char *) "-.0123456789", str.GetAt(*Pos) ))
	{
		(*Pos)++;
	}

	return num;
}



void CYahooTicker::ParseWebPage( CString& sTicker, CString& sDoc, int RetVal, CString& sErr )
{
	int Pos;

	// Reset string
	sTicker = "";

	// if error, text is error message
	if (RetVal)
	{
		sTicker = sErr;
		return;
	}

	// Time and date appended at transmit time

	// Check for alerts
	m_Beep = BeepType_NONE;
	m_Flash = FT_NONE;

	// Following line encodes left / right align info
	m_cflags = m_bLeft? 128 : 0;

	// Financials
	if (true)
	{
		CString sFin( sDoc );

		// remove &nbsp; etc
		ProcessHtml( sFin );

		// find stock
 		Pos = sFin.Find( m_sTriggerText );
		if (!m_bBefore) Pos+= m_sTriggerText.GetLength(); 

		if (Pos<0)
		{
			// say can't find info
			CString s;
			s.Format( IDS_CANTFINDSTOCK, (LPCTSTR)m_sTriggerText );
			sTicker += s;
		}
		else
		{
			// scan page to get all financial info
			double Num;

			// forward or back?
			for (int iCount = 0; iCount < this->m_iNumCount; iCount ++)
			{
				if (m_bBefore)
					Num = FindNumberBackwardsFrom( &Pos, sFin );
				else
					Num = FindNumberForwardsFrom( &Pos, sFin );
			}

			if (this->m_bCalcPercent)  // actually express as % of previous
			{
				double Num2 = FindNumberBackwardsFrom( &Pos, sFin );
				Num2 = FindNumberBackwardsFrom( &Pos, sFin );
				if (Num2!=0) 
					Num = Num / Num2 * 100;
				else
					Num = 0;
			}
			

				// Append data
			sTicker.Format( "%lf", Num );;
		
		}
	}

	return;

}

// ProcessHtml corrects all &nbsp; etc
void CYahooTicker::ProcessHtml( CString& Str )
{
	CString sTemp;
	int Pos;

	for (Pos=0; Pos<Str.GetLength(); Pos++)
	{
		if (Str.GetAt(Pos)=='&')
		{
			for (int i=0; i<NUMHTMLCODE; i++)
			{
				if (!Str.Mid(Pos, strlen(HTMLCode[i]) ).CompareNoCase("&nbsp;") )
				{
					sTemp += ReplaceChar[i];
					Pos += strlen(HTMLCode[i]) ;
					continue;
				}
			}
			continue;
		}

		// Ignore control characters
		if (Str[Pos]<32) continue;

		// Other text
		sTemp += Str[Pos];
	}

	Str = sTemp;
}

// Get next token finds the next separate word or string in quotes from position Pos
void CYahooTicker::GetNextToken( CString& Str, int Pos, CString& sTemp)
{
	// Find next word
	while (Str[Pos] == ' ') Pos++;
				
	// Text for quote
	if (Str[Pos] == '\"' )
	{
		Pos++;
		while (Str[Pos] != '\"' && Str[Pos] != '\0')
		{
			sTemp += Str[Pos];
			Pos++;
		}
	}
	else
	{
		while (Str[Pos] != ' ' && Str[Pos] != '\0')
		{
			sTemp += Str[Pos];
			Pos++;
		}
	}

	return ;
}

// Serialization
void CYahooTicker::Serialize(CArchive &ar)
{
	CDevice::Serialize( ar );

	if (ar.IsStoring())
	{
		ar << m_bChange;
		ar << m_bFinancial;
		ar << m_bHeadlines;
		ar << m_bLostSignal;
		ar << m_bMaxHead;
		ar << m_bNoZeroChange;
		ar << m_bOmitDigits;
		ar << m_bPercent;
		ar << m_bPrice;
		ar << m_bTime;
		ar << m_bWeather;
		ar << m_iMaxHead;
		ar << m_iNumLine;
		ar << m_strFirstLine;
		ar << m_TimeFormatString;
		ar << m_bLeft;
		ar << m_ScrollRate;
		ar << m_cflags;
		ar << (int) m_OldBeep;
		ar << m_BeepMin;
		ar << m_bCalcPercent ;
		ar << m_sTriggerText ;
		ar << m_bBefore ;
		ar << m_iNumCount ;
		ar << m_HiVal ;
		ar << m_HiDeg ;
		ar << m_LoDeg ;
		ar << m_LoVal ;
	}
	else
	{
		ar >> m_bChange;
		ar >> m_bFinancial;
		ar >> m_bHeadlines;
		ar >> m_bLostSignal;
		ar >> m_bMaxHead;
		ar >> m_bNoZeroChange;
		ar >> m_bOmitDigits;
		ar >> m_bPercent;
		ar >> m_bPrice;
		ar >> m_bTime;
		ar >> m_bWeather;
		ar >> m_iMaxHead;
		ar >> m_iNumLine;
		ar >> m_strFirstLine;
		ar >> m_TimeFormatString;
		ar >> m_bLeft;
		ar >> m_ScrollRate;
		m_slAlTrigger.RemoveAll();
		m_slAlMessage.RemoveAll();
		m_lbAlFlash.RemoveAll();
		m_lbAlFlag.RemoveAll();
		m_lbAlBeep.RemoveAll();
		ar >> m_cflags;
		m_cflags = 0;
		int i;
		ar >> i;
		m_OldBeep = (BeepType) i;;
		ar >> m_BeepMin;
		ar >> m_bCalcPercent ;
		ar >> m_sTriggerText ;
		ar >> m_bBefore ;
		ar >> m_iNumCount ;
		ar >> m_HiVal ;
		ar >> m_HiDeg ;
		ar >> m_LoDeg ;
		ar >> m_LoVal ;
	}

	m_slAlMessage.Serialize( ar );
	m_slAlTrigger.Serialize( ar );
	m_lbAlBeep.Serialize( ar );
	m_lbAlFlag.Serialize( ar );
	m_lbAlFlash.Serialize( ar );
}

void CYahooTicker::TransmitMessage()
{
	CHotOffThePressApp* pApp = (CHotOffThePressApp*)AfxGetApp();

	// See if data changed
	if (pApp->m_bDataUpdate)
	{
		pApp->m_bDataUpdate = false;

		// get data
		CCriticalSection cs;
		cs.Lock();
		m_TextTkrCopy = m_TickerText;
		cs.Unlock();

		double dval=0;
		sscanf( m_TextTkrCopy, "%lf", &dval );

		// calibrate
		if (dval < m_LoVal) dval = m_LoVal;
		if (dval > m_HiVal) dval = m_HiVal;
		dval = ( dval - m_LoVal ) / ( m_HiVal - m_LoVal ); // fraction 0 to 1
		dval = dval * ( ((double)m_HiDeg) - ((double)m_LoDeg) ) + ((double)m_LoDeg) + 0.5; // degrees
		dval = dval /2 ; // multiples of 2degrees
		char ival = (int) dval ;

		// Transmit - does not return until message complete
		DWORD nBW;
		if (pApp->m_hComm!=INVALID_HANDLE_VALUE && WriteFile( pApp->m_hComm, &ival, 1, &nBW, NULL) == 0)
		{
			AfxMessageBox( IDS_CANTTRANSMIT );
		}
	}

	return;
}

